# Functions for applying firewall with ipfilter

$ipf_file = "$module_config_directory/ipf.rules";

# check_firewall()
# Returns an error message if some firewall command is missing
sub check_firewall
{
return &has_command("ipf") ? undef :
	&text('iptables_missing', "<tt>ipf</tt>");
}

sub apply_services
{
# Create a rules file
open(SAVE, ">$ipf_file");

# Always allow localhost
print SAVE "pass in quick on lo from any to any keep state\n";

# Do all the services
local $s;
local $num = 1000;
foreach $s (&list_services()) {
	local $action = $s->{'allow'} ? "pass" : "block";
	local ($proto, $port);
	if ($s->{'serv'}) {
		$proto = $s->{'proto'};
		$port = $s->{'serv'};
		}
	else {
		$proto = "ip";
		}
	if ($s->{'mode'}) {
		# For some hosts .. add a rule for each
		foreach $h (split(/\s+/, $s->{'hosts'})) {
			print SAVE "$action in quick proto $proto from $h to any port = $port keep state\n";
			}
		}
	else {
		# For all hosts
		print SAVE "$action in quick proto $proto from any to any port = $port keep state\n";
		}
	$num += 10;
	}
print SAVE "pass in quick from any to any keep state\n";
print SAVE "pass out quick from any to any keep state\n";
close(SAVE);

# Clear existing rules
system("ipf -F a");

# Apply the rules file
local $out = `cd / ; ipf -f $ipf_file 2>&1`;
if ($?) {
	return "ipf output : <pre>$out</pre>";
	}
return undef;
}

# stop_services()
# Revert the firewall to accept-everything mode
sub stop_services
{
system("cd /; ipfw -f flush; ipfw add allow ip from any to any");
return undef;
}

1;

