# Functions for applying firewall with ipfw

$ipfw_file = "$module_config_directory/ipfw.sh";

# check_firewall()
# Returns an error message if some firewall command is missing
sub check_firewall
{
return &has_command("ipfw") ? undef :
	&text('iptables_missing', "<tt>ipfw</tt>");
}

sub apply_services
{
open(SAVE, ">$ipfw_file");
print SAVE "#!/bin/sh\n";
local $ipfw = &has_command("ipfw");

# Clear existing rules
print SAVE "$ipfw -f flush\n";

# Always allow established connections
print SAVE "$ipfw add 100 allow tcp from any to any established\n";

# Always allow localhost
print SAVE "$ipfw add 200 allow ip from any to any recv lo\n";

# Do all the services
local $s;
local $num = 1000;
foreach $s (&list_services()) {
	local $action = $s->{'allow'} ? "allow" : "deny";
	local ($proto, $port);
	if ($s->{'serv'}) {
		$proto = $s->{'proto'};
		$port = $s->{'serv'};
		}
	else {
		$proto = "ip";
		}
	if ($s->{'mode'}) {
		# For some hosts .. add a rule for each
		local $h;
		foreach $h (split(/\s+/, $s->{'hosts'})) {
			if (&check_ethernet($h)) {
				print SAVE "$ipfw add $num $action $proto from any to any $port mac any $h\n";
				}
			else {
				print SAVE "$ipfw add $num $action $proto from $h to any $port\n";
				}
			}
		}
	else {
		# For all hosts
		print SAVE "$ipfw add $num $action $proto from any to any $port\n";
		}
	$num += 10;
	}
print SAVE "$ipfw add 65000 allow ip from any to any\n";
close(SAVE);
chmod(755, $ipfw_file);

# Apply the save file
local $out = `cd / ; $ipfw_file 2>&1`;
if ($?) {
	return "ipfw output : <pre>$out</pre>";
	}
return undef;
}

# stop_services()
# Revert the firewall to accept-everything mode
sub stop_services
{
system("cd /; ipfw -f flush; ipfw add allow ip from any to any");
return undef;
}

1;

