# Functions for applying firewall with iptables

$iptables_file = "$module_config_directory/iptables";

# check_firewall()
# Returns an error message if some firewall command is missing
sub check_firewall
{
return &has_command("iptables-restore") ? undef :
	&text('iptables_missing', "<tt>iptables-restore</tt>");
}

sub apply_services
{
open(SAVE, ">$iptables_file");
print SAVE "*filter\n";
print SAVE ":INPUT ACCEPT [0:0]\n";
print SAVE ":OUTPUT ACCEPT [0:0]\n";
print SAVE ":FORWARD ACCEPT [0:0]\n";

# Always allow established connections
print SAVE "-A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT\n";
print SAVE "-A FORWARD -m state --state ESTABLISHED,RELATED -j ACCEPT\n";

# Always allow localhost
print SAVE "-A INPUT -i lo -j ACCEPT\n";
print SAVE "-A OUTPUT -o lo -j ACCEPT\n";

# Do all the services
local $s;
foreach $s (&list_services()) {
	local $rule;
	if ($s->{'serv'}) {
		$rule .= " -m ".$s->{'proto'};
		$rule .= " -p ".$s->{'proto'};
		$rule .= " --destination-port $s->{'serv'}";
		}
	if ($s->{'allow'}) {
		$rule .= " -j ACCEPT";
		}
	else {
		$rule .= " -j DROP";
		}
	if ($s->{'mode'}) {
		# For some hosts .. add a rule for each
		local $h;
		foreach $h (split(/\s+/, $s->{'hosts'})) {
			if (&check_ethernet($h)) {
				print SAVE "-A INPUT -m mac --mac-source $h $rule\n";
				print SAVE "-A FORWARD -m mac --mac-source $h $rule\n";
				}
			else {
				print SAVE "-A INPUT -s $h $rule\n";
				print SAVE "-A FORWARD -s $h $rule\n";
				}
			}
		}
	else {
		# For all hosts
		print SAVE "-A INPUT $rule\n";
		print SAVE "-A FORWARD $rule\n";
		}
	}
print SAVE "COMMIT\n";
close(SAVE);

# Apply the save file
local $out = `iptables-restore <$iptables_file 2>&1`;
if ($?) {
	return "iptables-restore output : <pre>$out</pre>";
	}
return undef;
}

# stop_services()
# Revert the firewall to accept-everything mode
sub stop_services
{
local $cmd;
foreach $cmd ("iptables -P INPUT ACCEPT",
	      "iptables -P OUTPUT ACCEPT",
	      "iptables -P FORWARD ACCEPT",
	      "iptables -F",
	      "iptables -X",
	      "iptables -Z") {
	local $out = `$cmd 2>&1`;
	if ($?) {
		return "$cmd output : $out";
		}
	}
return undef;
}

1;

